/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>
#include <stdio.h>
#include <pic16f18324.h>         /* For uint8_t definition */

#include "user.h"



/******************************************************************************/
/* User Functions                                                             */

/******************************************************************************/
/**
 * @brief enqueue
 * Insert new node in the rear of the queue
 * 
 * @param data Value to be inserted
 * @return 0 on success, otherwise - 1
 */
uint8_t enqueue(fifo_t *fifo, uint8_t data) {
    
    if (fifo->head == (fifo->tail + 1) % FIFO_SIZE) {
#if defined(__DEBUG__)
        printf("Queue overflow!\n\r");
#endif
        return 1;
    } else {
        /* Check if the buffer is empty */
        if (fifo->head == -1) {
            fifo->head = 0;
            fifo->tail = 0;
        } else {
            fifo->tail = (fifo->tail + 1) % FIFO_SIZE;
        }

        fifo->fifo[fifo->tail] = data;
//        registers.tx_fifo[tail] = data;
    }
    return 0;
}

/**
 * @brief dequeue
 * Delete first item
 * 
 * @return 0 on fail, otherwise data
 */
uint8_t dequeue(fifo_t *fifo) {
    uint8_t data;
    /* Check if queue is empty */
    if ((fifo->head == fifo->tail) && (fifo->tail == -1)) {
#if defined(__DEBUG__)
        printf("Queue underflow!\n\r");
#endif
        return 0;
    } else {
        /* Get the data*/
        data = fifo->fifo[fifo->head];

        /* Delete node */
        if (fifo->head == fifo->tail) {
            fifo->head = -1;
            fifo->tail = -1;
        } else {
            /* Increment front */
            fifo->head = (fifo->head + 1) % FIFO_SIZE;
        }        
    }
#if defined(__DEBUG__)
    printf("Returning: %d\n\r", data);
#endif
    return data;
}

/**
 * @brief queue_available
 * Check queue status
 * 
 * @return  0 if there are no bytes available, otherwise 1
 */
uint8_t queue_available(fifo_t *fifo)
{
    if(fifo->head == -1)
        return 0;
    else
        return 1;
}